from requests import Response, get, post
from abc import ABC, abstractmethod
from types import TracebackType
from typing import IO, Any, Self, final


class Connector(ABC):
    @property
    @abstractmethod
    def connected(self) -> bool:
        raise NotImplementedError

    @abstractmethod
    def connect(self) -> None:
        raise NotImplementedError

    @abstractmethod
    def disconnect(self) -> None:
        raise NotImplementedError

    @final
    def __enter__(self) -> Self:
        self.connect()
        return self

    @final
    def __exit__(
        self,
        _exc_type: type[BaseException] | None,
        _exc_value: BaseException | None,
        _exc_tb: TracebackType | None,
    ) -> bool:
        self.disconnect()
        return False


class RESTConnector(Connector, ABC):
    url: str
    port: int

    def post(
        self,
        endpoint: str,
        data: dict[str, Any],
        files: dict[str, IO] | None = None,
        header: dict[str, Any] | None = None,
    ) -> Response:
        return post(
            f"https://{self.url}:{self.port}/{endpoint}",
            data=data,
            files=files,
            headers=header,
        )

    def get(self, endpoint: str, data: dict[str, Any]) -> Response:
        return get(f"https://{self.url}:{self.port}/{endpoint}", params=data)
